/* Parche para modificar el ejecutable de XCOM: UFO Defense Gold Edition
	Tambin arregla el error de las ees en biglets.dat y smallset.dat  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

const char *copyright = "= Preparado por Hector Gustavo Gimenez para www.dantoine.com =";

/* Cadenas del ejecutable */
const char *english = "english";
const char *spanish = "spanish";
const char *englishu = "ENGLISH";
const char *espanyol = "ESPAOL";
char *orglang,*newlang;

/* ufoexe apunta al ejecutable */
FILE *ufoexe;

/* rutinas de error */
int fail(char *error,char *filename,FILE *filefail)
{
	printf("Se produjo un error al intentar %s ",error);
	if (strcmpi(error,"abrir") != 0)
		printf("en ");
	printf("\"%s\"\n",filename);
	if (filefail != NULL)
		fclose(filefail);
	return 1;
}
/* cpyfont copia definiciones de las letras dentro de un mismo archivo
	orgfont: 	ubicacin de la letra correcta.
	destfont:	ubicacin de la letra a corregir.
	fontsize:	tamao en bytes de la letra.
	fontname:	el archivo a modificar. */
int cpyfont(int orgfont,int destfont,int fontsize,const char *fontname)
{
	char buffer[256];
	FILE *fontfile = fopen(fontname,"rb+");
	if (fontfile == NULL)
		return fail("abrir",fontname,NULL);
	if (fseek(fontfile,fontsize*orgfont,SEEK_SET) != 0)
		return fail("buscar",fontname,fontfile);
	if (fgets(buffer,fontsize+1,fontfile) == NULL)
		return fail("leer",fontname,fontfile);
	if (fseek(fontfile,fontsize*destfont,SEEK_SET) != 0)
		return fail("buscar",fontname,fontfile);
	if (fwrite(buffer,1,fontsize,fontfile) != fontsize)
		return fail("escribir",fontname,fontfile);
	fclose(fontfile);
	return 0;
}
/* wrtexe modifica el ejecutable
	offset: posicin en el archivo donde se realizar la modificacin	*/
int wrtexe(long int offset,char *exename)
{
	/* qu casualidad!, todas las cadenas tienen 7 bytes ;) */
	char buffer7[7];
	if (fseek(ufoexe,offset,SEEK_SET) != 0)
		return fail("buscar",exename,NULL);
	if (fgets(buffer7,8,ufoexe) == NULL)
		return fail("leer",exename,NULL);
	if (strcmp(buffer7,orglang) == 0) {
		if (fseek(ufoexe,offset,SEEK_SET) != 0)
			return fail("buscar",exename,NULL);
		if (fwrite(newlang,1,7,ufoexe) != 7)
			return fail("escribir",exename,NULL);
	}
	else
		if (strcmp(buffer7,newlang) != 0) {
			printf(
				"Offset %x: No se encuentra la palabra clave %s en \"%s\"\n"
				,offset,orglang,exename);
			return 1;
		}
	return 0;
}
/* wrtnames modifica los nombres de los soldados en el ejecutable */
int wrtnames(char *filename, long int offset)
{
	FILE *names = fopen(filename, "rb");
	if (names == NULL)
		return fail("abrir",filename,NULL);
	fseek(names,0,SEEK_END);
	int size = ftell(names);
	char *buffer = (char *)malloc(size);
	fseek(names,0,SEEK_SET);
	if (fread(buffer,1,size,names) != size)
		return fail("leer",filename,names);
	fseek(ufoexe,offset,SEEK_SET);
	if (fwrite(buffer,1,size,ufoexe) != size)
		return fail("escribir",filename,names);
	fclose(names);
	return 0;
}
/********************* Ac empieza la cosa. ********************/
int main(int argc,char *argv[])
{
	/* si se llama al programa sin argumentos */
	if (argc == 1) {
		printf(
			"Argumentos insuficientes.\n"
			"Sintaxis:\n"
			"			parche \"ejecutable\" [english]\n"
			"			parche fix\n\n"
			"Este programa modifica el ejecutable para mostrar un botn con la palabra\n"
			"ESPAOL y usar los archivos de lenguaje \"spanish.dat\" y \"spanish2.dat\"\n"
			"Usar como argumento el nombre del ejecutable. Para revertir los cambios\n"
			"agregar english como ltimo argumento. Ej.: \"parche ufo.exe english\"\n"
			"Por ltimo, \"parche fix\"\n arregla el problema de las ees en los \n"
			"archivos \"biglets.dat\" y \"smallset.dat\"\n"
			"NOTA: La modificacin de los nombres de los soldados no es reversible.\n\n"
			);
		return 1;
	}
	/* si se arregla el bug de las ees, modificando los archivos de las
	fuentes. En biglets.dat, la ee mayscula se copia de la posicin 149
    a la 132, que se corresponde con ASCII 165. En smallset.dat, la ee
    minscula se copia de 119 a 131 (ASCII 164). En el primero cada letra
	ocupa 256 bytes (16x16) y en el segundo, 72 bytes (8x9). */
	if (strcmpi(argv[1],"fix") == 0) {
		if (cpyfont(149,132,256,".\\geodata\\biglets.dat")
		|| cpyfont(119,131,72,".\\geodata\\smallset.dat"))
			return 1;
		printf("\"Biglets.dat\" y \"smallset.dat\" modificados.\n\n");
		return 0;
	}
	/* si se desea modificar el ejecutable */
	ufoexe = fopen(argv[1],"rb+");
	if (ufoexe == NULL)
		return fail("abrir",argv[1],NULL);
	/* se comprueba si es una reversin */
	if ((argv[2] != NULL) && (strcmpi(argv[2],english) == 0)) {
		orglang = spanish;
		newlang = english;
	}
	else {
		orglang = english;
		newlang = spanish;
	}
	/* se modifican las referencias a los *ish?.dat */
	if (wrtexe(0x6c984,argv[1]) || wrtexe(0x742d0,argv[1]))
		exit (1);
	if (orglang == english) {
		orglang = englishu;
		newlang = espanyol;
	}
	else {
		orglang = espanyol;
		newlang = englishu;
	}
	/* se modifica el texto del botn */
	if (wrtexe(0x74298,argv[1]))
		exit(1);
	/* Se modifican los nombres y apellidos a menos que se intente una
	restauracin. No se comprueban los errores. */
	if (orglang == englishu) {
		if (wrtnames("apellidos.dat",0x6e6e8) == 0)
			printf("Apellidos modificados.\n");
		if (wrtnames("nombres.dat",0x6ec88) == 0)
			printf("Nombres modificados.\n");
	}
	fclose(ufoexe);
	/* todo listo, nos despedimos */
	if (orglang == englishu) {
		printf(
			"\"%s\" modificado.\n"
			"Para revertir los cambios usa: parche \"%s\" english\n\n"
			,argv[1],argv[1]);
	}
	else
		printf("\"%s\" restaurado.\n\n",argv[1]);
	return 0;
}
/* Eso es todo */
